@@ -41,9 +41,7 @@
 #include "champlain-defines.h"
 #include "champlain-marshal.h"
 #include "champlain-private.h"
-#include "champlain-map.h"
 #include "champlain-tile.h"
-#include "champlain-zoom-level.h"
 
 #include <clutter/clutter.h>
 #include <glib.h>
@@ -71,14 +69,21 @@ G_DEFINE_TYPE (ChamplainBaseMarker, champlain_base_marker, CLUTTER_TYPE_GROUP);
 
 #define CHAMPLAIN_BASE_MARKER_GET_PRIVATE(obj)    (G_TYPE_INSTANCE_GET_PRIVATE((obj), CHAMPLAIN_TYPE_BASE_MARKER, ChamplainBaseMarkerPrivate))
 
+struct _ChamplainBaseMarkerPrivate
+{
+  gdouble lon;
+  gdouble lat;
+  gboolean highlighted;
+};
+
 static void
 champlain_base_marker_get_property (GObject *object,
-                               guint prop_id,
-                               GValue *value,
-                               GParamSpec *pspec)
+    guint prop_id,
+    GValue *value,
+    GParamSpec *pspec)
 {
     ChamplainBaseMarker *base_marker = CHAMPLAIN_BASE_MARKER (object);
-    ChamplainBaseMarkerPrivate *priv = CHAMPLAIN_BASE_MARKER_GET_PRIVATE (base_marker);
+    ChamplainBaseMarkerPrivate *priv = base_marker->priv;
 
     switch (prop_id)
       {
@@ -98,12 +103,12 @@ champlain_base_marker_get_property (GObject *object,
 
 static void
 champlain_base_marker_set_property (GObject *object,
-                               guint prop_id,
-                               const GValue *value,
-                               GParamSpec *pspec)
+    guint prop_id,
+    const GValue *value,
+    GParamSpec *pspec)
 {
     ChamplainBaseMarker *base_marker = CHAMPLAIN_BASE_MARKER (object);
-    ChamplainBaseMarkerPrivate *priv = CHAMPLAIN_BASE_MARKER_GET_PRIVATE (base_marker);
+    ChamplainBaseMarkerPrivate *priv = base_marker->priv;
 
     switch (prop_id)
     {
@@ -130,22 +135,26 @@ champlain_base_marker_set_property (GObject *object,
     }
 }
 
+static void
+champlain_base_marker_dispose (GObject *object)
+{
+  G_OBJECT_CLASS (champlain_base_marker_parent_class)->dispose (object);
+}
+
 static void
 champlain_base_marker_finalize (GObject *object)
 {
-  //ChamplainBaseMarker *base_marker = CHAMPLAIN_BASE_MARKER (object);
-  //ChamplainBaseMarkerPrivate *priv = CHAMPLAIN_BASE_MARKER_GET_PRIVATE (base_marker);
-
   G_OBJECT_CLASS (champlain_base_marker_parent_class)->finalize (object);
 }
 
 static void
-champlain_base_marker_class_init (ChamplainBaseMarkerClass *champlainBaseMarkerClass)
+champlain_base_marker_class_init (ChamplainBaseMarkerClass *marker_class)
 {
-  g_type_class_add_private (champlainBaseMarkerClass, sizeof (ChamplainBaseMarkerPrivate));
+  g_type_class_add_private (marker_class, sizeof (ChamplainBaseMarkerPrivate));
 
-  GObjectClass *object_class = G_OBJECT_CLASS (champlainBaseMarkerClass);
+  GObjectClass *object_class = G_OBJECT_CLASS (marker_class);
   object_class->finalize = champlain_base_marker_finalize;
+  object_class->dispose = champlain_base_marker_dispose;
   object_class->get_property = champlain_base_marker_get_property;
   object_class->set_property = champlain_base_marker_set_property;
 
@@ -190,7 +199,9 @@ static void
 champlain_base_marker_init (ChamplainBaseMarker *marker)
 {
   ChamplainBaseMarkerPrivate *priv = CHAMPLAIN_BASE_MARKER_GET_PRIVATE (marker);
+
   marker->priv = priv;
+
   priv->lat = 0;
   priv->lon = 0;
   priv->highlighted = FALSE;
@@ -200,6 +211,8 @@ champlain_base_marker_init (ChamplainBaseMarker *marker)
 /**
  * champlain_base_marker_new:
  *
+ * Creates a new instance of #ChamplainBaseMarker.
+ *
  * Returns: a new #ChamplainBaseMarker ready to be used as a #ClutterActor.
  *
  * Since: 0.4
@@ -210,7 +223,6 @@ champlain_base_marker_new (void)
   ChamplainBaseMarker *base_marker;
 
   base_marker = CHAMPLAIN_BASE_MARKER (g_object_new (CHAMPLAIN_TYPE_BASE_MARKER, NULL));
-  //ChamplainBaseMarkerPrivate *priv = CHAMPLAIN_BASE_MARKER_GET_PRIVATE (base_marker);
 
   return CLUTTER_ACTOR (base_marker);
 }
@@ -226,19 +238,53 @@ champlain_base_marker_new (void)
  * Since: 0.4
  */
 void
-champlain_base_marker_set_position (ChamplainBaseMarker *champlainBaseMarker,
+champlain_base_marker_set_position (ChamplainBaseMarker *marker,
     gdouble latitude,
     gdouble longitude)
 {
-  g_return_if_fail (CHAMPLAIN_IS_BASE_MARKER (champlainBaseMarker));
+  g_return_if_fail (CHAMPLAIN_IS_BASE_MARKER (marker));
 
-  ChamplainBaseMarkerPrivate *priv = CHAMPLAIN_BASE_MARKER_GET_PRIVATE (champlainBaseMarker);
+  ChamplainBaseMarkerPrivate *priv = marker->priv;
 
   priv->lon = longitude;
   priv->lat = latitude;
 
-  g_object_notify (G_OBJECT (champlainBaseMarker), "latitude");
-  g_object_notify (G_OBJECT (champlainBaseMarker), "longitude");
+  g_object_notify (G_OBJECT (marker), "latitude");
+  g_object_notify (G_OBJECT (marker), "longitude");
+}
+
+/**
+ * champlain_base_marker_get_latitude:
+ * @marker: a #ChamplainBaseMarker
+ *
+ * Gets the latitude of the marker.
+ *
+ * Returns: the latitude of the marker.
+ *
+ * Since: 0.6
+ */
+gdouble champlain_base_marker_get_latitude (ChamplainBaseMarker *marker)
+{
+  g_return_val_if_fail (CHAMPLAIN_IS_BASE_MARKER (marker), 0.0);
+
+  return marker->priv->lat;
+}
+
+/**
+ * champlain_base_marker_get_longitude:
+ * @marker: a #ChamplainBaseMarker
+ *
+ * Gets the longitude of the marker.
+ *
+ * Returns: the longitude of the marker.
+ *
+ * Since: 0.6
+ */
+gdouble champlain_base_marker_get_longitude (ChamplainBaseMarker *marker)
+{
+  g_return_val_if_fail (CHAMPLAIN_IS_BASE_MARKER (marker), 0.0);
+
+  return marker->priv->lon;
 }
 
 /**
@@ -252,34 +298,32 @@ champlain_base_marker_set_position (ChamplainBaseMarker *champlainBaseMarker,
  * Since: 0.4
  */
 void
-champlain_base_marker_set_highlighted (ChamplainBaseMarker *champlainBaseMarker,
+champlain_base_marker_set_highlighted (ChamplainBaseMarker *marker,
     gboolean value)
 {
-  g_return_if_fail (CHAMPLAIN_IS_BASE_MARKER (champlainBaseMarker));
+  g_return_if_fail (CHAMPLAIN_IS_BASE_MARKER (marker));
 
-  ChamplainBaseMarkerPrivate *priv = CHAMPLAIN_BASE_MARKER_GET_PRIVATE (champlainBaseMarker);
+  marker->priv->highlighted = value;
 
-  priv->highlighted = value;
-
-  g_object_notify (G_OBJECT (champlainBaseMarker), "highlighted");
+  g_object_notify (G_OBJECT (marker), "highlighted");
 }
 
 /**
  * champlain_base_marker_get_highlighted:
  * @marker: a #ChamplainBaseMarker
  *
+ * Checks whether the marker is highlighted.
+ *
  * Returns: the highlighted or not state of the marker.
  *
  * Since: 0.4
  */
 gboolean
-champlain_base_marker_get_highlighted (ChamplainBaseMarker *champlainBaseMarker)
+champlain_base_marker_get_highlighted (ChamplainBaseMarker *marker)
 {
-  g_return_val_if_fail (CHAMPLAIN_IS_BASE_MARKER (champlainBaseMarker), FALSE);
+  g_return_val_if_fail (CHAMPLAIN_IS_BASE_MARKER (marker), FALSE);
 
-  ChamplainBaseMarkerPrivate *priv = CHAMPLAIN_BASE_MARKER_GET_PRIVATE (champlainBaseMarker);
-
-  return priv->highlighted;
+  return marker->priv->highlighted;
 }
 
 /**
@@ -310,7 +354,6 @@ void
 champlain_base_marker_animate_in_with_delay (ChamplainBaseMarker *marker,
     guint delay)
 {
-  ClutterAnimation *animation;
   ClutterTimeline *timeline;
   gfloat y;
 
@@ -324,10 +367,9 @@ champlain_base_marker_animate_in_with_delay (ChamplainBaseMarker *marker,
 
   timeline = clutter_timeline_new (1000);
   clutter_timeline_set_delay (timeline, delay);
-  animation = clutter_actor_animate_with_timeline (CLUTTER_ACTOR (marker),
+  clutter_actor_animate_with_timeline (CLUTTER_ACTOR (marker),
       CLUTTER_EASE_OUT_BOUNCE, timeline, "opacity", 255, "y", y,
       "scale-x", 1.0, "scale-y", 1.0, NULL);
-  timeline = clutter_animation_get_timeline (animation);
 }
 
 /**
@@ -358,10 +400,13 @@ on_idle (ChamplainBaseMarker* marker)
 }
 
 static void
-on_animation_completed (ClutterAnimation* animation,
+on_animation_completed (G_GNUC_UNUSED ClutterAnimation* animation,
     ChamplainBaseMarker *marker)
 {
-  g_idle_add ((GSourceFunc) on_idle, marker);
+  g_idle_add_full (G_PRIORITY_DEFAULT,
+                   (GSourceFunc)on_idle,
+                   g_object_ref (marker),
+                   (GDestroyNotify)g_object_unref);
 }
 
 /**
@@ -392,7 +437,6 @@ champlain_base_marker_animate_out_with_delay (ChamplainBaseMarker *marker,
   animation = clutter_actor_animate_with_timeline (CLUTTER_ACTOR (marker),
       CLUTTER_EASE_IN_BACK, timeline, "opacity", 0, "y", y - 100,
       "scale-x", 2.0, "scale-y", 2.0, NULL);
-  timeline = clutter_animation_get_timeline (animation);
   g_signal_connect (animation, "completed",
       G_CALLBACK (on_animation_completed), marker);
 }
