@@ -17,6 +17,20 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
+/**
+ * SECTION:champlain-network-tile-source
+ * @short_description: A map source that downloads tiles from a web server
+ *
+ * This object is specialized for map tiles that can be downloaded
+ * from a web server.  This includes all web based map services such as
+ * OpenStreetMap, Google Maps, Yahoo Maps and more.  This object contains
+ * all mechanisms necessary to download tiles.
+ *
+ * Some preconfigured network map sources are built-in this library,
+ * see #ChamplainMapSourceFactory.
+ *
+ */
+
 #include "config.h"
 
 #include "champlain-network-tile-source.h"
@@ -30,7 +44,6 @@
 #include "champlain-map-source.h"
 #include "champlain-marshal.h"
 #include "champlain-private.h"
-#include "champlain-zoom-level.h"
 
 #include <errno.h>
 #include <gdk/gdk.h>
@@ -59,8 +72,6 @@ G_DEFINE_TYPE (ChamplainNetworkTileSource, champlain_network_tile_source, CHAMPL
 
 #define GET_PRIVATE(obj)    (G_TYPE_INSTANCE_GET_PRIVATE((obj), CHAMPLAIN_TYPE_NETWORK_TILE_SOURCE, ChamplainNetworkTileSourcePrivate))
 
-typedef struct _ChamplainNetworkTileSourcePrivate ChamplainNetworkTileSourcePrivate;
-
 struct _ChamplainNetworkTileSourcePrivate
 {
   gboolean offline;
@@ -75,14 +86,20 @@ typedef struct
   ChamplainTile *tile;
 } TileLoadedCallbackData;
 
+typedef struct
+{
+  ChamplainMapSource *map_source;
+  SoupMessage *msg;
+} TileDestroyedCbData;
+
 static void fill_tile (ChamplainMapSource *map_source,
                        ChamplainTile *tile);
 
 static gchar *
 get_tile_uri (ChamplainNetworkTileSource *source,
-              gint x,
-              gint y,
-              gint z);
+    gint x,
+    gint y,
+    gint z);
 
 static void
 champlain_network_tile_source_get_property (GObject *object,
@@ -90,7 +107,7 @@ champlain_network_tile_source_get_property (GObject *object,
     GValue *value,
     GParamSpec *pspec)
 {
-  ChamplainNetworkTileSourcePrivate *priv = GET_PRIVATE(object);
+  ChamplainNetworkTileSourcePrivate *priv = CHAMPLAIN_NETWORK_TILE_SOURCE (object)->priv;
 
   switch (prop_id)
     {
@@ -132,13 +149,17 @@ champlain_network_tile_source_set_property (GObject *object,
     }
 }
 
+
 static void
 champlain_network_tile_source_dispose (GObject *object)
 {
-  ChamplainNetworkTileSourcePrivate *priv = GET_PRIVATE(object);
+  ChamplainNetworkTileSourcePrivate *priv = CHAMPLAIN_NETWORK_TILE_SOURCE (object)->priv;
 
   if (priv->soup_session)
+  {
     soup_session_abort (priv->soup_session);
+    priv->soup_session = NULL;
+  }
 
   G_OBJECT_CLASS (champlain_network_tile_source_parent_class)->dispose (object);
 }
@@ -146,7 +167,7 @@ champlain_network_tile_source_dispose (GObject *object)
 static void
 champlain_network_tile_source_finalize (GObject *object)
 {
-  ChamplainNetworkTileSourcePrivate *priv = GET_PRIVATE(object);
+  ChamplainNetworkTileSourcePrivate *priv = CHAMPLAIN_NETWORK_TILE_SOURCE(object)->priv;
 
   g_free (priv->uri_format);
   g_free (priv->proxy_uri);
@@ -177,25 +198,46 @@ champlain_network_tile_source_class_init (ChamplainNetworkTileSourceClass *klass
 
   map_source_class->fill_tile = fill_tile;
 
+  /**
+  * ChamplainNetworkTileSource:uri-format
+  *
+  * The uri format of the tile source, see #champlain_network_tile_source_set_uri_format
+  *
+  * Since: 0.4
+  */
   pspec = g_param_spec_string ("uri-format",
-                               "URI Format",
-                               "The URI format",
-                               "",
-                               (G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
+      "URI Format",
+      "The URI format",
+      "",
+      (G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
   g_object_class_install_property (object_class, PROP_URI_FORMAT, pspec);
 
+  /**
+  * ChamplainNetworkTileSource:offline
+  *
+  * Specifies whether the network tile source can access network
+  *
+  * Since: 0.4
+  */
   pspec = g_param_spec_boolean ("offline",
-                                "Offline",
-                                "Offline",
-                                FALSE,
-                                G_PARAM_READWRITE);
+      "Offline",
+      "Offline",
+      FALSE,
+      G_PARAM_READWRITE);
   g_object_class_install_property (object_class, PROP_OFFLINE, pspec);
 
+  /**
+  * ChamplainNetworkTileSource:proxy-uri
+  *
+  * The proxy uri used to access network
+  *
+  * Since: 0.4
+  */
   pspec = g_param_spec_string ("proxy-uri",
-                               "Proxy URI",
-                               "The proxy URI to use to access network",
-                               "",
-                               G_PARAM_READWRITE);
+      "Proxy URI",
+      "The proxy URI to use to access network",
+      "",
+      G_PARAM_READWRITE);
   g_object_class_install_property (object_class, PROP_PROXY_URI, pspec);
 }
 
@@ -204,6 +246,8 @@ champlain_network_tile_source_init (ChamplainNetworkTileSource *tile_source)
 {
   ChamplainNetworkTileSourcePrivate *priv = GET_PRIVATE (tile_source);
 
+  tile_source->priv = priv;
+
   priv->proxy_uri = NULL;
   priv->uri_format = NULL;
   priv->offline = FALSE;
@@ -215,47 +259,90 @@ champlain_network_tile_source_init (ChamplainNetworkTileSource *tile_source)
 #endif
                          NULL);
   g_object_set (G_OBJECT (priv->soup_session),
-                "user-agent", "libchamplain/" CHAMPLAIN_VERSION_S,
-                "max-conns-per-host", 2, NULL); // This is as required by OSM
+      "user-agent", "libchamplain/" CHAMPLAIN_VERSION_S,
+      "max-conns-per-host", 2, NULL); // This is as required by OSM
 
 }
 
+/**
+ * champlain_network_tile_source_new_full:
+ * @id: the map source's id
+ * @name: the map source's name
+ * @license: the map source's license
+ * @license_uri: the map source's license URI
+ * @min_zoom: the map source's minimum zoom level
+ * @max_zoom: the map source's maximum zoom level
+ * @tile_size: the map source's tile size (in pixels)
+ * @projection: the map source's projection
+ * @uri_format: the URI to fetch the tiles from, see #champlain_network_tile_source_set_uri_format
+ *
+ * Constructor of #ChamplainNetworkTileSource.
+ *
+ * Returns: a constructed #ChamplainNetworkTileSource
+ *
+ * Since: 0.4
+ */
 ChamplainNetworkTileSource*
 champlain_network_tile_source_new_full (const gchar *id,
-                                        const gchar *name,
-                                        const gchar *license,
-                                        const gchar *license_uri,
-                                        guint min_zoom,
-                                        guint max_zoom,
-                                        guint tile_size,
-                                        ChamplainMapProjection projection,
-                                        const gchar *uri_format)
+    const gchar *name,
+    const gchar *license,
+    const gchar *license_uri,
+    guint min_zoom,
+    guint max_zoom,
+    guint tile_size,
+    ChamplainMapProjection projection,
+    const gchar *uri_format)
 {
   ChamplainNetworkTileSource * source;
   source = g_object_new (CHAMPLAIN_TYPE_NETWORK_TILE_SOURCE, "id", id,
-                         "name", name, "license", license, "license-uri", license_uri,
-                         "min-zoom-level", min_zoom, "max-zoom-level", max_zoom,
-                         "tile-size", tile_size, "projection", projection,
-                         "uri-format", uri_format, NULL);
+      "name", name, "license", license, "license-uri", license_uri,
+      "min-zoom-level", min_zoom, "max-zoom-level", max_zoom,
+      "tile-size", tile_size, "projection", projection,
+      "uri-format", uri_format, NULL);
   return source;
 }
 
+/**
+ * champlain_network_tile_source_get_uri_format:
+ * @tile_source: the #ChamplainNetworkTileSource
+ *
+ * Default constructor of #ChamplainNetworkTileSource.
+ *
+ * Returns: A URI format used for URI creation when downloading tiles. See
+ * champlain_network_tile_source_set_uri_format() for more information.
+ *
+ * Since: 0.6
+ */
 const gchar *
 champlain_network_tile_source_get_uri_format (ChamplainNetworkTileSource *tile_source)
 {
   g_return_val_if_fail (CHAMPLAIN_IS_NETWORK_TILE_SOURCE (tile_source), NULL);
 
-  ChamplainNetworkTileSourcePrivate *priv = GET_PRIVATE(tile_source);
-  return priv->uri_format;
+  return tile_source->priv->uri_format;
 }
 
+/**
+ * champlain_network_tile_source_set_uri_format:
+ * @tile_source: the #ChamplainNetworkTileSource
+ * @uri_format: the URI format
+ *
+ * A URI format is a URI where x, y and zoom level information have been
+ * marked for parsing and insertion.  There can be an unlimited number of
+ * marked items in a URI format.  They are delimited by "#" before and after
+ * the variable name. There are 3 defined variable names: X, Y, and Z.
+ *
+ * For example, this is the OpenStreetMap URI format:
+ * "http://tile.openstreetmap.org/\#Z\#/\#X\#/\#Y\#.png"
+ *
+ * Since: 0.4
+ */
 void
 champlain_network_tile_source_set_uri_format (ChamplainNetworkTileSource *tile_source,
     const gchar *uri_format)
 {
   g_return_if_fail (CHAMPLAIN_IS_NETWORK_TILE_SOURCE (tile_source));
 
-  ChamplainNetworkTileSourcePrivate *priv = GET_PRIVATE(tile_source);
+  ChamplainNetworkTileSourcePrivate *priv = tile_source->priv;
 
   g_free (priv->uri_format);
   priv->uri_format = g_strdup (uri_format);
@@ -263,22 +350,40 @@ champlain_network_tile_source_set_uri_format (ChamplainNetworkTileSource *tile_s
   g_object_notify (G_OBJECT (tile_source), "uri-format");
 }
 
+/**
+ * champlain_network_tile_source_get_proxy_uri:
+ * @tile_source: the #ChamplainNetworkTileSource
+ *
+ * Gets the proxy uri used to access network.
+ *
+ * Returns: the proxy uri
+ *
+ * Since: 0.6
+ */
 const gchar *
 champlain_network_tile_source_get_proxy_uri (ChamplainNetworkTileSource *tile_source)
 {
   g_return_val_if_fail (CHAMPLAIN_IS_NETWORK_TILE_SOURCE (tile_source), NULL);
 
-  ChamplainNetworkTileSourcePrivate *priv = GET_PRIVATE(tile_source);
-  return priv->proxy_uri;
+  return tile_source->priv->proxy_uri;
 }
 
+/**
+ * champlain_network_tile_source_set_proxy_uri:
+ * @tile_source: the #ChamplainNetworkTileSource
+ * @proxy_uri: the proxy uri used to access network
+ *
+ * Sets the proxy uri used to access network.
+ *
+ * Since: 0.6
+ */
 void
 champlain_network_tile_source_set_proxy_uri (ChamplainNetworkTileSource *tile_source,
     const gchar *proxy_uri)
 {
   g_return_if_fail (CHAMPLAIN_IS_NETWORK_TILE_SOURCE (tile_source));
 
-  ChamplainNetworkTileSourcePrivate *priv = GET_PRIVATE(tile_source);
+  ChamplainNetworkTileSourcePrivate *priv = tile_source->priv;
   SoupURI *uri = NULL;
 
   g_free (priv->proxy_uri);
@@ -293,41 +398,57 @@ champlain_network_tile_source_set_proxy_uri (ChamplainNetworkTileSource *tile_so
                   NULL);
 
   if (uri)
-    g_object_unref (uri);
+    soup_uri_free (uri);
 
   g_object_notify (G_OBJECT (tile_source), "proxy-uri");
 }
 
+/**
+ * champlain_network_tile_source_get_offline:
+ * @tile_source: the #ChamplainNetworkTileSource
+ *
+ * Gets offline status.
+ *
+ * Returns: TRUE when the tile source is set to be offline; FALSE otherwise.
+ *
+ * Since: 0.6
+ */
 gboolean
 champlain_network_tile_source_get_offline (ChamplainNetworkTileSource *tile_source)
 {
   g_return_val_if_fail (CHAMPLAIN_IS_NETWORK_TILE_SOURCE (tile_source), FALSE);
 
-  ChamplainNetworkTileSourcePrivate *priv = GET_PRIVATE(tile_source);
-  return priv->offline;
+  return tile_source->priv->offline;
 }
 
+/**
+ * champlain_network_tile_source_set_offline:
+ * @tile_source: the #ChamplainNetworkTileSource
+ * @offline: TRUE when the tile source should be offline; FALSE otherwise
+ *
+ * Sets offline status.
+ *
+ * Since: 0.6
+ */
 void
 champlain_network_tile_source_set_offline (ChamplainNetworkTileSource *tile_source,
     gboolean offline)
 {
   g_return_if_fail (CHAMPLAIN_IS_NETWORK_TILE_SOURCE (tile_source));
 
-  ChamplainNetworkTileSourcePrivate *priv = GET_PRIVATE(tile_source);
-
-  priv->offline = offline;
+  tile_source->priv->offline = offline;
 
   g_object_notify (G_OBJECT (tile_source), "offline");
 }
 
 #define SIZE 8
 static gchar *
-get_tile_uri (ChamplainNetworkTileSource *source,
-              gint x,
-              gint y,
-              gint z)
+get_tile_uri (ChamplainNetworkTileSource *tile_source,
+    gint x,
+    gint y,
+    gint z)
 {
-  ChamplainNetworkTileSourcePrivate *priv = GET_PRIVATE(source);
+  ChamplainNetworkTileSourcePrivate *priv = tile_source->priv;
 
   gchar **tokens;
   gchar *token;
@@ -369,9 +490,32 @@ get_tile_uri (ChamplainNetworkTileSource *source,
 }
 
 static void
-tile_loaded_cb (SoupSession *session,
-                SoupMessage *msg,
-                gpointer user_data)
+tile_destroyed_cb (G_GNUC_UNUSED ChamplainTile *tile,
+    TileDestroyedCbData *data)
+{
+  if (data->map_source && data->msg)
+    {
+      DEBUG ("Canceling tile download");
+      ChamplainNetworkTileSourcePrivate *priv = CHAMPLAIN_NETWORK_TILE_SOURCE (data->map_source)->priv;
+
+      soup_session_cancel_message (priv->soup_session, data->msg, SOUP_STATUS_CANCELLED);
+    }
+}
+
+static void
+destroy_cb_data (TileDestroyedCbData *data,
+    G_GNUC_UNUSED GClosure *closure)
+{
+  if (data->map_source)
+    g_object_remove_weak_pointer(G_OBJECT (data->map_source), (gpointer*)&data->map_source);
+
+  g_free (data);
+}
+
+static void
+tile_loaded_cb (G_GNUC_UNUSED SoupSession *session,
+    SoupMessage *msg,
+    gpointer user_data)
 {
   TileLoadedCallbackData *callback_data = (TileLoadedCallbackData *)user_data;
   ChamplainMapSource *map_source = callback_data->map_source;
@@ -379,22 +523,26 @@ tile_loaded_cb (SoupSession *session,
   ChamplainTileCache *tile_cache = champlain_tile_source_get_cache (tile_source);
   ChamplainMapSource *next_source = champlain_map_source_get_next_source (map_source);
   ChamplainTile *tile = callback_data->tile;
-  GdkPixbufLoader* loader;
+  GdkPixbufLoader* loader = NULL;
   GError *error = NULL;
   ClutterActor *actor;
   const gchar *etag;
 
+  if (tile)
+    g_object_remove_weak_pointer (G_OBJECT (tile), (gpointer*)&callback_data->tile);
+
   g_free (user_data);
 
   DEBUG ("Got reply %d", msg->status_code);
 
-  if (msg->status_code == SOUP_STATUS_CANCELLED)
+  if (!tile || msg->status_code == SOUP_STATUS_CANCELLED)
     {
-      DEBUG ("Download of tile %d, %d got cancelled",
-             champlain_tile_get_x (tile), champlain_tile_get_y (tile));
-      //champlain_tile_set_state (tile, CHAMPLAIN_STATE_DONE);
+      if (!tile)
+        DEBUG ("Tile destroyed while loading");
+      else
+        DEBUG ("Download of tile %d, %d got cancelled",
+               champlain_tile_get_x (tile), champlain_tile_get_y (tile));
 
-      g_object_unref (tile);
       g_object_unref (map_source);
       return;
     }
@@ -429,7 +577,7 @@ tile_loaded_cb (SoupSession *session,
           g_error_free (error);
         }
 
-      goto load_next_cleanup;
+      goto load_next;
     }
 
   gdk_pixbuf_loader_close (loader, &error);
@@ -437,7 +585,7 @@ tile_loaded_cb (SoupSession *session,
     {
       g_warning ("Unable to close the pixbuf loader: %s", error->message);
       g_error_free (error);
-      goto load_next_cleanup;
+      goto load_next;
     }
 
   /* Verify if the server sent an etag and save it */
@@ -454,14 +602,14 @@ tile_loaded_cb (SoupSession *session,
   GdkPixbuf* pixbuf = gdk_pixbuf_loader_get_pixbuf (loader);
   actor = clutter_texture_new ();
   if (!clutter_texture_set_from_rgb_data (CLUTTER_TEXTURE (actor),
-                                          gdk_pixbuf_get_pixels (pixbuf),
-                                          gdk_pixbuf_get_has_alpha (pixbuf),
-                                          gdk_pixbuf_get_width (pixbuf),
-                                          gdk_pixbuf_get_height (pixbuf),
-                                          gdk_pixbuf_get_rowstride (pixbuf),
-                                          gdk_pixbuf_get_bits_per_sample (pixbuf) *
-                                          gdk_pixbuf_get_n_channels (pixbuf) / 8,
-                                          0, &error))
+           gdk_pixbuf_get_pixels (pixbuf),
+           gdk_pixbuf_get_has_alpha (pixbuf),
+           gdk_pixbuf_get_width (pixbuf),
+           gdk_pixbuf_get_height (pixbuf),
+           gdk_pixbuf_get_rowstride (pixbuf),
+           gdk_pixbuf_get_bits_per_sample (pixbuf) *
+           gdk_pixbuf_get_n_channels (pixbuf) / 8,
+           0, &error))
     {
       if (error)
         {
@@ -470,41 +618,61 @@ tile_loaded_cb (SoupSession *session,
         }
 
       g_object_unref (actor);
-      goto load_next_cleanup;
+      goto load_next;
     }
 
-  champlain_tile_set_content (tile, actor, TRUE);
-  champlain_tile_set_size (tile, champlain_map_source_get_tile_size (map_source));
+  champlain_tile_set_fade_in (tile, TRUE);
+  champlain_tile_set_content (tile, actor);
 
   goto finish;
 
-load_next_cleanup:
-  g_object_unref (loader);
-
 load_next:
+  if (loader)
+    g_object_unref (loader);
   if (next_source)
-    {
-      champlain_map_source_fill_tile (next_source, tile);
-    }
-  g_object_unref (tile);
+    champlain_map_source_fill_tile (next_source, tile);
   g_object_unref (map_source);
   return;
 
 finish:
+  if (loader)
+    g_object_unref (loader);
   champlain_tile_set_state (tile, CHAMPLAIN_STATE_DONE);
-  g_object_unref (tile);
   g_object_unref (map_source);
 }
 
+static gchar *
+get_modified_time_string (ChamplainTile *tile)
+{
+  const GTimeVal *time;
+  g_return_val_if_fail (CHAMPLAIN_TILE (tile), NULL);
+
+  time = champlain_tile_get_modified_time(tile);
+
+  if (time == NULL)
+    return NULL;
+
+  struct tm *other_time = gmtime (&time->tv_sec);
+  char value [100];
+
+#ifdef G_OS_WIN32
+  strftime (value, 100, "%a, %d %b %Y %H:%M:%S %Z", other_time);
+#else
+  strftime (value, 100, "%a, %d %b %Y %T %Z", other_time);
+#endif
+
+  return g_strdup (value);
+}
+
 static void
 fill_tile (ChamplainMapSource *map_source,
-           ChamplainTile *tile)
+    ChamplainTile *tile)
 {
   g_return_if_fail (CHAMPLAIN_IS_NETWORK_TILE_SOURCE (map_source));
   g_return_if_fail (CHAMPLAIN_IS_TILE (tile));
 
   ChamplainNetworkTileSource *tile_source = CHAMPLAIN_NETWORK_TILE_SOURCE (map_source);
-  ChamplainNetworkTileSourcePrivate *priv = GET_PRIVATE(tile_source);
+  ChamplainNetworkTileSourcePrivate *priv = tile_source->priv;
 
   if (!priv->offline)
     {
@@ -524,7 +692,7 @@ fill_tile (ChamplainMapSource *map_source,
           /* validate tile */
 
           const gchar *etag = champlain_tile_get_etag (tile);
-          gchar *date = champlain_tile_get_modified_time_string (tile);
+          gchar *date = get_modified_time_string (tile);
 
           /* If an etag is available, only use it.
            * OSM servers seems to send now as the modified time for all tiles
@@ -534,31 +702,38 @@ fill_tile (ChamplainMapSource *map_source,
             {
               DEBUG("If-None-Match: %s", etag);
               soup_message_headers_append (msg->request_headers,
-                                           "If-None-Match", etag);
+                  "If-None-Match", etag);
             }
           else if (date)
             {
               DEBUG("If-Modified-Since %s", date);
               soup_message_headers_append (msg->request_headers,
-                                           "If-Modified-Since", date);
+                  "If-Modified-Since", date);
             }
 
           g_free (date);
         }
 
+      TileDestroyedCbData *tile_destroyed_cb_data = g_new (TileDestroyedCbData, 1);
+      tile_destroyed_cb_data->map_source = map_source;
+      tile_destroyed_cb_data->msg = msg;
+
+      g_object_add_weak_pointer (G_OBJECT (msg), (gpointer*)&tile_destroyed_cb_data->msg);
+      g_object_add_weak_pointer (G_OBJECT (map_source), (gpointer*)&tile_destroyed_cb_data->map_source);
+
+      g_signal_connect_data (tile, "destroy", G_CALLBACK (tile_destroyed_cb),
+          tile_destroyed_cb_data, (GClosureNotify) destroy_cb_data, 0);
+
       callback_data = g_new (TileLoadedCallbackData, 1);
       callback_data->tile = tile;
       callback_data->map_source = map_source;
 
-      /* Ref the tile as it may be freeing during the loading
-       * Unref when the loading is done.
-       */
-      g_object_ref (tile);
+      g_object_add_weak_pointer (G_OBJECT (tile), (gpointer*)&callback_data->tile);
       g_object_ref (map_source);
 
       soup_session_queue_message (priv->soup_session, msg,
-                                  tile_loaded_cb,
-                                  callback_data);
+          tile_loaded_cb,
+          callback_data);
     }
   else
     {
