@@ -29,6 +29,7 @@
 #define COL_NAME 1
 
 static ChamplainPolygon *polygon;
+static gboolean destroying = FALSE;
 
 /*
  * Terminate the main loop.
@@ -36,6 +37,7 @@ static ChamplainPolygon *polygon;
 static void
 on_destroy (GtkWidget *widget, gpointer data)
 {
+  destroying = TRUE;
   gtk_main_quit ();
 }
 
@@ -84,15 +86,10 @@ map_source_changed (GtkWidget *widget,
   gtk_tree_model_get (model, &iter, COL_ID, &id, -1);
 
   ChamplainMapSourceFactory *factory = champlain_map_source_factory_dup_default ();
-  source = champlain_map_source_factory_create (factory, id);
+  source = champlain_map_source_factory_create_cached_source (factory, id);
 
-  if (source != NULL)
-    {
-      g_object_set (G_OBJECT (view), "map-source", source, NULL);
-      g_object_unref (factory);
-    }
-
-  g_object_unref (source);
+  g_object_set (G_OBJECT (view), "map-source", source, NULL);
+  g_object_unref (factory);
 }
 
 static void
@@ -120,16 +117,17 @@ view_state_changed (ChamplainView *view,
 {
   ChamplainState state;
 
+  if (destroying)
+    return;
+
   g_object_get (G_OBJECT (view), "state", &state, NULL);
   if (state == CHAMPLAIN_STATE_LOADING)
     {
       gtk_image_set_from_stock (image, GTK_STOCK_NETWORK, GTK_ICON_SIZE_BUTTON);
-      g_print("STATE: loading\n");
     }
   else
     {
       gtk_image_clear (image);
-      g_print("STATE: done\n");
     }
 }
 
@@ -277,6 +275,8 @@ main (int argc,
   gtk_container_add (GTK_CONTAINER (bbox), button);
 
   button = gtk_spin_button_new_with_range(0, 20, 1);
+  gtk_spin_button_set_value (GTK_SPIN_BUTTON (button),
+      champlain_view_get_zoom_level (view));
   g_signal_connect (button, "changed", G_CALLBACK (zoom_changed), view);
   g_signal_connect (view, "notify::zoom-level", G_CALLBACK (map_zoom_changed),
       button);
